%%% ------------------------------------------------------------------
%%% Standard utilities

append([], L, L).
append([X | Xs], Ys, [X | Zs])  :- append(Xs, Ys, Zs).

between(I, M, N) :-
	M < N,
	I = M.
between(I, M, N) :-
	M < N,
	M1 is M+1,
	between(I, M1,  N).





%% ph(N, OutFile) writes formula for N objects, N-1 holes, with encoding
%	(Radix*Obj + Hole) for variable to mean Obj is in Hole.
%	Radix is stored in radix(Radix) for flexibility.

radix(1000).

ph(N, OutFile)  :-
	( N > 0  ->
		tell(OutFile),
		negatives(N),
		positives(N),
		told
	;
		write('N (arg 1) must be positive, not this: '(N)), nl
	).


negatives(N)  :-
	N1 is N + 1,
	radix(Radix),
	between(Hole, 1, N),
	between(Obj1, 1, N1),
	Next is Obj1 + 1,
	between(Obj2, Next, N1),
	V1 is Radix*Obj1 + Hole,
	V2 is Radix*Obj2 + Hole,
	write('(-'), write(V1), write(', -'), write(V2), write(').'), nl,
	fail.
negatives(_).

positives(N)  :-
	N1 is N + 1,
	radix(Radix),
	between(Obj1, 1, N1),
	write('('),
	Hole = 1,
	V1 is Radix*Obj1 + Hole,
	( N > 1  -> write(V1); write(false) ),
	positive(Obj1, N),
	write(').'), nl,
	fail.
positives(_).

positive(Obj1, N)  :-
	radix(Radix),
	between(Hole, 2, N),
	V1 is Radix*Obj1 + Hole,
	write(', '), write(V1),
	fail.
positive(_, _).

